/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.conqat.lib.commons.collections.UnmodifiableMap;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.ITutorialProvider;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.extension.data.TutorialStep;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.service.ITutorialService;
import org.fortiss.tooling.kernel.service.listener.ITutorialServiceListener;

/**
 * This class implements the {@link ITutorialService} interface.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public final class TutorialService implements ITutorialService {

	/** The collection of all defined tutorials. */
	private Map<String, Class<? extends ITutorialProvider>> allTutorials = new HashMap<>();
	/** The collection of listeners. */
	private Collection<ITutorialServiceListener> listeners =
			new LinkedList<ITutorialServiceListener>();

	/** The active tutorial. */
	private ITutorialProvider activeTutorial = null;
	/** The active tutorial step. */
	private TutorialStep activeStep = null;

	/** {@inheritDoc} */
	@Override
	public boolean isTutorialActive() {
		return activeTutorial != null;
	}

	/** {@inheritDoc} */
	@Override
	public ITutorialProvider getActiveTutorial() {
		return activeTutorial;
	}

	/** {@inheritDoc} */
	@Override
	public TutorialStep getActiveTutorialStep() {
		return activeStep;
	}

	/** {@inheritDoc} */
	@Override
	public Map<String, Class<? extends ITutorialProvider>> getAllTutorialProviders() {
		return new UnmodifiableMap<String, Class<? extends ITutorialProvider>>(allTutorials);
	}

	/** {@inheritDoc} */
	@Override
	public void registerTutorialProvider(Class<? extends ITutorialProvider> provider,
			String displayName) {
		allTutorials.put(displayName, provider);
	}

	/** {@inheritDoc} */
	@Override
	public void startTutorial(Class<? extends ITutorialProvider> provider) {
		if(!allTutorials.containsValue(provider)) {
			return;
		}
		try {
			activeTutorial = provider.newInstance();
			activeStep = activeTutorial.getInitialStep();
			EObject root = activeTutorial.getRootElement();
			IPersistencyService.INSTANCE.addDummyEObjectAsTopLevelElement(root);
		} catch(InstantiationException e) {
			e.printStackTrace();
		} catch(IllegalAccessException e) {
			e.printStackTrace();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void stopTutorial() {
		IPersistencyService.INSTANCE.removeDummyTopLevelElement(activeTutorial.getRootElement());
		activeStep = null;
		activeTutorial = null;
	}

	/** {@inheritDoc} */
	@Override
	public void addTutorialServiceListener(ITutorialServiceListener l) {
		if(!listeners.contains(l)) {
			listeners.add(l);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void removeTutorialServiceListener(ITutorialServiceListener l) {
		listeners.remove(l);
	}

	/** {@inheritDoc} */
	@Override
	public boolean prototypeActive(Prototype prototype) {
		return activeTutorial.getGlobalWhitelistProvider().prototypeActive(prototype) ||
				activeStep.getWhitelistProvider().prototypeActive(prototype);
	}
}
